#ifndef AirportPropertyEditor_h
#define AirportPropertyEditor_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <Transactions/TransactionObjectChangeListener.h>

#include <wx/listctrl.h>
#include <wx/scrolwin.h>

class Airport;

class AirportPropertyEditor: public wxScrolledWindow,
                             public TransactionObjectChangeListener {
        
        DECLARE_CLASS(AirportPropertyEditor)
        
public:
	AirportPropertyEditor();
	AirportPropertyEditor(wxWindow* parent,
                                Airport *airport,
                                wxWindowID id = -1,
                                const wxPoint& pos = wxDefaultPosition,
                                const wxSize& size = wxDefaultSize,
                                long style = wxHSCROLL | wxVSCROLL,
                                const wxString& name = wxT("airportPropertyEditor"));
        virtual ~AirportPropertyEditor();
        bool Create(wxWindow* parent,
                                Airport *airport,
                                wxWindowID id = -1,
                                const wxPoint& pos = wxDefaultPosition,
                                const wxSize& size = wxDefaultSize,
                                long style = wxHSCROLL | wxVSCROLL,
                                const wxString& name = wxT("airportPropertyEditor"));
        
        void OnTextEnter(wxCommandEvent& evt);
        void OnCheckbox(wxCommandEvent& evt);
        void OnFrequencySelected(wxListEvent& evt);
        void OnFrequencyDeselected(wxListEvent& evt);
        void OnAddFrequency(wxCommandEvent& evt);
        void OnUpdateFrequency(wxCommandEvent& evt);
        void OnRemoveFrequency(wxCommandEvent& evt);

        virtual void BeforeObjectChange(TransactionObject* object);
        virtual void AfterObjectChange(TransactionObject* object);
protected:
        void UpdateFields();
        
        Airport *_airport;
        
        wxTextCtrl *_icaoID;
        wxTextCtrl *_name;
        wxTextCtrl *_elevation;
        wxCheckBox *_hasControlTower;
        wxCheckBox *_hasDefaultBuildings;
        wxListCtrl *_frequencyList;
        wxChoice *_frequencyType;
        wxTextCtrl *_frequency;
        wxTextCtrl *_frequencyName;
        wxButton *_addFrequency;
        wxButton *_updateFrequency;
        wxButton *_removeFrequency;

        wxArrayString _frequencyTypeChoices;
private:
        DECLARE_EVENT_TABLE()
};

#endif

