#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "RunwaySelection.h"
#include "RunwayMapObject.h"
#include <Map/MapCoordinateSystem.h>
#include <Map/MapEditor.h>
#include <Map/MapProjection.h>
#include <Map/MapSelection.h>
#include <Map/MapSelectionManager.h>

#include "PavementSelectionEndHandle.h"
#include "RunwaySelectionLengthHandle.h"

IMPLEMENT_CLASS(RunwaySelection,PavementSelection);

RunwaySelection::RunwaySelection(MapSelectionManager* manager, RunwayMapObject* content):
        PavementSelection(manager,content) {
}

RunwaySelection::~RunwaySelection() {
}

MapSelectionHandle* RunwaySelection::HitHandle(Point3D p) {
        if (!HandlesActive())
                return (MapSelectionHandle*)NULL;
        MapEditor* editor=GetSelectionManager()->GetEditor();
        MapProjection* proj=editor->GetProjection();
        MapCoordinateSystem* cs=&editor->GetCoordinateSystem();
        
        // FIXME: The PavementSelectionEndHandle does not properly change the length in presence of stopways
        Point3D rectsize=GetHandleBoxLogicalSize();
        for (unsigned int i=0;i<2;i++) {
                Point3D endpoint=proj->ConvertToLocal(GetContent()->GetEndCenters()[i]);
                Point3D delta=p-endpoint;
                if (endpoint.distance3Dsquared(p)*4.0<(rectsize.x()*rectsize.x())) {
                        return new PavementSelectionEndHandle(this,i);
                }
        }
        
        double dl=5,length,width;
        int side;
        
        length=GetContent()->GetLength();
        width=GetContent()->GetWidth();
        if (dl>length/2)
                dl=length*0.9/2;
        if (dl>width/2)
                dl=width*0.9/2;
        side=GetHitSide(p,GetContent()->GetEndCorners(),dl);
        if (side==0 || side==2) {
                return new RunwaySelectionLengthHandle(this,side/2,RunwaySelectionLengthHandle::EndType_RunwayEnd);
        }
        side=GetHitSide(p,GetContent()->GetStopwayCorners(),dl);
        if (side==0 || side==2) {
                return new RunwaySelectionLengthHandle(this,side/2,RunwaySelectionLengthHandle::EndType_StopwayEnd);
        }
        side=GetHitSide(p,GetContent()->GetThresholdCorners(),dl);
        if (side==0 || side==2) {
                return new RunwaySelectionLengthHandle(this,side/2,RunwaySelectionLengthHandle::EndType_Threshold);
        }
        
        return PavementSelection::HitHandle(p);
}

void RunwaySelection::ProvidePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        if (!targetClass->IsKindOf(CLASSINFO(RunwaySelection))) {
                PavementSelection::ProvidePropertiesForClass(propgrid,targetClass);
                return;
        }
        
        propgrid->Append(wxStringProperty(wxT("Designation"),wxT("RunwaySelection_Designation")));
        propgrid->Append(wxBoolProperty(wxT("Distance Remaining Signs"),wxT("RunwaySelection_DistanceRemainingSigns")));
        
        wxPGChoices shoulderChoices;
        shoulderChoices.Add(wxT("None"),RunwayMapObject::Shoulder_None);
        shoulderChoices.Add(wxT("Asphalt"),RunwayMapObject::Shoulder_Asphalt);
        shoulderChoices.Add(wxT("Concrete"),RunwayMapObject::Shoulder_Concrete);
        
        propgrid->Append(wxEnumProperty(wxT("Shoulder"),
                                        wxT("RunwaySelection_Shoulder"),
                                        shoulderChoices));
        
        wxPGChoices markingChoices;
        markingChoices.Add(wxT("None"),RunwayMapObject::Markings_None);
        markingChoices.Add(wxT("Visual"),RunwayMapObject::Markings_Visual);
        markingChoices.Add(wxT("Non-precision"),RunwayMapObject::Markings_NonPrecision);
        markingChoices.Add(wxT("Precision"),RunwayMapObject::Markings_Precision);
        
        propgrid->Append(wxEnumProperty(wxT("Approach Markings"),
                                        wxT("RunwaySelection_Markings"),
                                        markingChoices));

        PavementSelection::ProvidePropertiesForClass(propgrid,targetClass);

        wxPGChoices surfaceChoices;
        surfaceChoices.Add(wxT("Runway: Asphalt"),PavementMapObject::Surface_Asphalt);
        surfaceChoices.Add(wxT("Runway: Concrete"),PavementMapObject::Surface_Concrete);
        surfaceChoices.Add(wxT("Runway: Turf"),PavementMapObject::Surface_Turf);
        surfaceChoices.Add(wxT("Runway: Dirt"),PavementMapObject::Surface_Dirt);
        surfaceChoices.Add(wxT("Runway: Gravel"),PavementMapObject::Surface_Gravel);
        surfaceChoices.Add(wxT("Runway: Dry Lakebed"),PavementMapObject::Surface_Lakebed);
        surfaceChoices.Add(wxT("Runway: Water"),PavementMapObject::Surface_Water);
        surfaceChoices.Add(wxT("Helipad: Asphalt"),PavementMapObject::Surface_Helipad_Asphalt);
        surfaceChoices.Add(wxT("Helipad: Concrete"),PavementMapObject::Surface_Helipad_Concrete);
        surfaceChoices.Add(wxT("Helipad: Turf"),PavementMapObject::Surface_Helipad_Turf);
        surfaceChoices.Add(wxT("Helipad: Dirt"),PavementMapObject::Surface_Helipad_Dirt);
        
        propgrid->SetPropertyChoices(wxT("PavementSelection_Surface"),surfaceChoices);

        propgrid->AppendCategory(wxT("Designated Runway End"));
        ProvideRunwayEndProperties(propgrid,
                        wxT("RunwaySelection_RunwayEndA"));
        
        propgrid->AppendCategory(wxT("Opposite Runway End"));
        ProvideRunwayEndProperties(propgrid,
                        wxT("RunwaySelection_RunwayEndB"));
}

void RunwaySelection::ProvideRunwayEndProperties(wxPropertyGrid* propgrid,
                                                 const wxString idPrefix) {
        propgrid->Append(wxIntProperty(wxT("Displaced Threshold Length"),idPrefix+wxT("_DisplacedThreshold")));
        propgrid->Append(wxIntProperty(wxT("Stopway Length"),idPrefix+wxT("_StopwayLength")));
        propgrid->Append(wxFloatProperty(wxT("VASI Angle (deg)"),idPrefix+wxT("_VASIAngle")));
        
        wxPGChoices VAPLChoices;
        VAPLChoices.Add(wxT("None"),(int)RunwayMapObject::VAPL_None);
        VAPLChoices.Add(wxT("VASI"),(int)RunwayMapObject::VAPL_VASI);
        VAPLChoices.Add(wxT("PAPI"),(int)RunwayMapObject::VAPL_PAPI);
        VAPLChoices.Add(wxT("Space Shuttle PAPI (20deg)"),(int)RunwayMapObject::VAPL_SpaceShuttlePAPI);
        
        propgrid->Append(wxEnumProperty(wxT("Visual Approach Path Lightning"),
                                        idPrefix+wxT("_VAPL"),
                                        VAPLChoices));
        
        wxPGChoices RWLChoices;
        RWLChoices.Add(wxT("None"),(int)RunwayMapObject::RWYL_None);
        RWLChoices.Add(wxT("White Edge Lights"),(int)RunwayMapObject::RWYL_EdgeLights);
        RWLChoices.Add(wxT("REIL"),(int)RunwayMapObject::RWYL_REIL);
        RWLChoices.Add(wxT("CLL"),(int)RunwayMapObject::RWYL_CLL);
        RWLChoices.Add(wxT("TDZ"),(int)RunwayMapObject::RWYL_TDZ);
        
        propgrid->Append(wxEnumProperty(wxT("Runway Lightning"),
                                        idPrefix+wxT("_RWL"),
                                        RWLChoices));
        
        wxPGChoices APPLChoices;
        APPLChoices.Add(wxT("None"),(int)RunwayMapObject::APPL_None);
        APPLChoices.Add(wxT("SSALS"),(int)RunwayMapObject::APPL_SSALS);
        APPLChoices.Add(wxT("SALSF"),(int)RunwayMapObject::APPL_SALSF);
        APPLChoices.Add(wxT("ALSF I"),(int)RunwayMapObject::APPL_ALSF_I);
        APPLChoices.Add(wxT("ALSF II"),(int)RunwayMapObject::APPL_ALSF_II);
        APPLChoices.Add(wxT("ODALS"),(int)RunwayMapObject::APPL_ODALS);
        APPLChoices.Add(wxT("Calvert 1"),(int)RunwayMapObject::APPL_Calvert_1);
        APPLChoices.Add(wxT("Calvert 2 and 3"),(int)RunwayMapObject::APPL_Calvert_2_3);
        
        propgrid->Append(wxEnumProperty(wxT("Approach Lightning"),
                                        idPrefix+wxT("_APPL"),
                                        APPLChoices));
}

void RunwaySelection::UpdatePropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass) {
        PavementSelection::UpdatePropertiesForClass(propgrid,targetClass);
        if (!targetClass->IsKindOf(CLASSINFO(RunwaySelection)))
                return;

        propgrid->SetPropertyValue(wxT("RunwaySelection_Designation"),GetContent()->GetDesignation());
        propgrid->SetPropertyValue(wxT("RunwaySelection_DistanceRemainingSigns"),GetContent()->HasDistanceRemainingSigns());
        propgrid->SetPropertyValue(wxT("RunwaySelection_Shoulder"),(int)GetContent()->GetShoulder());
        propgrid->SetPropertyValue(wxT("RunwaySelection_Markings"),(int)GetContent()->GetMarkings());
        
        UpdateRunwayEndProperties(propgrid,
                        wxT("RunwaySelection_RunwayEndA"),
                        GetContent()->GetDesignatedEnd());
        
        UpdateRunwayEndProperties(propgrid,
                        wxT("RunwaySelection_RunwayEndB"),
                        GetContent()->GetOppositeEnd());
}

void RunwaySelection::UpdateRunwayEndProperties(wxPropertyGrid* propgrid,
                                 const wxString idPrefix,
                                 RunwayMapObject::RunwayEndData rwyEndData) {
        propgrid->SetPropertyValue(idPrefix+wxT("_DisplacedThreshold"),rwyEndData.displacedThreshold);
        propgrid->SetPropertyValue(idPrefix+wxT("_StopwayLength"),rwyEndData.stopwayLength);
        propgrid->SetPropertyValue(idPrefix+wxT("_VASIAngle"),rwyEndData.vasiAngle);
        propgrid->SetPropertyValue(idPrefix+wxT("_VAPL"),rwyEndData.visualApproachPathLighting);
        propgrid->SetPropertyValue(idPrefix+wxT("_RWL"),rwyEndData.runwayLighting);
        propgrid->SetPropertyValue(idPrefix+wxT("_APPL"),rwyEndData.approachLighting);
}

void RunwaySelection::MarkDifferingPropertiesForClass(wxPropertyGrid* propgrid, wxClassInfo* targetClass, MapSelection* firstSelection) {
        PavementSelection::MarkDifferingPropertiesForClass(propgrid,targetClass,firstSelection);
        if (!targetClass->IsKindOf(CLASSINFO(RunwaySelection)))
                return;
        RunwayMapObject* firstContent=wxDynamicCast(firstSelection,RunwaySelection)->GetContent();
        RunwayMapObject* myContent=GetContent();

        if (firstContent->GetDesignation().Cmp(myContent->GetDesignation())!=0)
                propgrid->SetPropertyUnspecified(wxT("RunwaySelection_Designation"));
        if (firstContent->HasDistanceRemainingSigns()!=myContent->HasDistanceRemainingSigns())
                propgrid->SetPropertyUnspecified(wxT("RunwaySelection_DistanceRemainingSigns"));
        if (firstContent->GetShoulder()!=myContent->GetShoulder())
                propgrid->SetPropertyUnspecified(wxT("RunwaySelection_Shoulder"));
        if (firstContent->GetMarkings()!=myContent->GetMarkings())
                propgrid->SetPropertyUnspecified(wxT("RunwaySelection_Markings"));

        MarkDifferingRunwayEndProperties(propgrid,
                        wxT("RunwaySelection_RunwayEndA"),
                        myContent->GetDesignatedEnd(),
                        firstContent->GetDesignatedEnd());
        MarkDifferingRunwayEndProperties(propgrid,
                        wxT("RunwaySelection_RunwayEndB"),
                        myContent->GetOppositeEnd(),
                        firstContent->GetOppositeEnd());
}

void RunwaySelection::MarkDifferingRunwayEndProperties(wxPropertyGrid* propgrid,
                                        const wxString idPrefix,
                                        RunwayMapObject::RunwayEndData myRwyEndData,
                                        RunwayMapObject::RunwayEndData firstRwyEndData) {
        if (myRwyEndData.displacedThreshold!=firstRwyEndData.displacedThreshold)
                propgrid->SetPropertyUnspecified(idPrefix+wxT("_DisplacedThreshold"));
        if (myRwyEndData.stopwayLength!=firstRwyEndData.stopwayLength)
                propgrid->SetPropertyUnspecified(idPrefix+wxT("_StopwayLength"));
        if (fabs(myRwyEndData.vasiAngle-firstRwyEndData.vasiAngle)>=0.01)
                propgrid->SetPropertyUnspecified(idPrefix+wxT("_VASIAngle"));
        if (myRwyEndData.visualApproachPathLighting!=firstRwyEndData.visualApproachPathLighting)
                propgrid->SetPropertyUnspecified(idPrefix+wxT("_VAPL"));
        if (myRwyEndData.runwayLighting!=firstRwyEndData.runwayLighting)
                propgrid->SetPropertyUnspecified(idPrefix+wxT("_RWL"));
        if (myRwyEndData.approachLighting!=firstRwyEndData.approachLighting)
                propgrid->SetPropertyUnspecified(idPrefix+wxT("_APPL"));
}

void RunwaySelection::UpdateObjectFromProperty(wxPGProperty* property) {
        const wxString& name=property->GetName();
        RunwayMapObject::RunwayEndData designatedEnd,oppositeEnd;
        
        designatedEnd=GetContent()->GetDesignatedEnd();
        oppositeEnd=GetContent()->GetOppositeEnd();

        if (name.Cmp(wxT("RunwaySelection_Designation"))==0) {
                GetContent()->SetDesignation(property->GetValueAsString());
        } else if (name.Cmp(wxT("RunwaySelection_DistanceRemainingSigns"))==0) {
                GetContent()->SetDistanceRemainingSigns(property->GetValueAsVariant().GetBool());
        } else if (name.Cmp(wxT("RunwaySelection_Shoulder"))==0) {
                GetContent()->SetShoulder((RunwayMapObject::Shoulder)property->GetValueAsVariant().GetLong());
        } else if (name.Cmp(wxT("RunwaySelection_Markings"))==0) {
                GetContent()->SetMarkings((RunwayMapObject::Markings)property->GetValueAsVariant().GetLong());
        } else if (UpdateRunwayEndFromProperty(property,
                                wxT("RunwaySelection_RunwayEndA"),
                                designatedEnd)) {
                GetContent()->SetDesignatedEnd(designatedEnd);
        } else if (UpdateRunwayEndFromProperty(property,
                                wxT("RunwaySelection_RunwayEndB"),
                                oppositeEnd)) {
                GetContent()->SetOppositeEnd(oppositeEnd);
        } else {
                PavementSelection::UpdateObjectFromProperty(property);
        }
}

bool RunwaySelection::UpdateRunwayEndFromProperty(wxPGProperty* property,
                                 const wxString idPrefix,
                                 RunwayMapObject::RunwayEndData& rwyEndData) {
        const wxString& name=property->GetName();
        if (!name.StartsWith(idPrefix))
                return false;
        wxString subname=name.Mid(idPrefix.Length());
        wxVariant value=property->GetValueAsVariant();
        
        if (subname.Cmp(wxT("_DisplacedThreshold"))==0) {
                rwyEndData.displacedThreshold=value.GetLong();
        } else if (subname.Cmp(wxT("_StopwayLength"))==0) {
                rwyEndData.stopwayLength=value.GetLong();
        } else if (subname.Cmp(wxT("_VASIAngle"))==0) {
                rwyEndData.vasiAngle=value.GetDouble();
        } else if (subname.Cmp(wxT("_VAPL"))==0) {
                rwyEndData.visualApproachPathLighting=(RunwayMapObject::VisualApproachPathLighting)value.GetLong();
        } else if (subname.Cmp(wxT("_RWL"))==0) {
                rwyEndData.runwayLighting=(RunwayMapObject::RunwayLighting)value.GetLong();
        } else if (subname.Cmp(wxT("_APPL"))==0) {
                rwyEndData.approachLighting=(RunwayMapObject::ApproachLighting)value.GetLong();
        } else 
                return false;
        return true;
}

