#ifndef MapSimpleBackgroundImage_h
#define MapSimpleBackgroundImage_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <wx/image.h>
#include <Map/MapObject.h>

class MapSimpleBackgroundImage: public MapObject {
        
        DECLARE_CLASS(MapSimpleBackgroundImage)
        TRANSACTIONOBJECT_DEFINE_SMARTREF(MapSimpleBackgroundImage)
        
public:
	MapSimpleBackgroundImage(TransactionManager* transactionManager,
                                 Point3D northWestPos,
                                 double scale);
        
        virtual ~MapSimpleBackgroundImage();
        
        virtual void Paint(wxDC& dc, MapEditor* editor, const MapPass& pass);
        
        virtual bool Hit(MapEditor* editor, TDRectangle r);
        
        virtual MapSelection* CreateSelection(MapSelectionManager* manager);
        
        virtual wxRect GetDeviceBounds(MapEditor* editor);
        
        virtual TDRectangle GetGeographicBounds();
        
        TRANSACTIONOBJECT_DEFINE_STATE_GETTER(const wxString&,Filename,_fileName);
        void SetFilename(const wxString& v) {
                if (TRANSACTIONOBJECT_STATEVAR(_fileName).Cmp(v)==0)
                        return;
                BeforeWrite();
                
                TRANSACTIONOBJECT_STATEVAR(_fileName)=v;
                LoadImage();
                
                AfterWrite();
        }

        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(Point3D,NorthWestPos,_northWestPos)
        TRANSACTIONOBJECT_DEFINE_STATE_ACCESSOR(double,Scale,_scale)

        virtual const wxImage& LoadImage();
        virtual void GetLocalCorners(MapEditor* editor, Point3D corners[4]);
        virtual void GetDeviceCorners(MapEditor* editor, wxPoint corners[4]);
protected:
        typedef struct {
                wxString _fileName;
                Point3D _northWestPos;
                double _scale;
        } State;
        
        State _myState;
       
        class Memento: public MapObject::Memento {
                friend class MapSimpleBackgroundImage;
        public:
                Memento(const MapSimpleBackgroundImage& obj):
                        MapObject::Memento(obj)
                {
                        _savedState=obj._myState;
                }
        protected:
                State _savedState;
        };
public:
        virtual Memento* CreateMemento() {
                return new Memento(*this);
        }
        virtual void RestoreMemento(wxObject* object) {
                MapObject::RestoreMemento(object);
                Memento* memento=dynamic_cast<Memento*>(object);
                wxASSERT(memento);
                
                bool filenameDiffers;
                filenameDiffers=(_myState._fileName.Cmp(memento->_savedState._fileName)!=0);
               
                _myState=memento->_savedState;
                
                if (filenameDiffers) {
                        LoadImage();
                }
        }
        
protected:
        virtual const wxImage& GetImage() { return _image; }
        virtual const wxPoint GetImageOffset() { return wxPoint(0,0); }

        wxImage _image;
};

#endif

