#include "MapUSGSImageLayer.h"

#include "MapUSGSImage.h"

#include "USGSFetcher.h"

IMPLEMENT_CLASS(MapUSGSImageLayer,MapObjectContainer);

UTMProjection MapUSGSImageLayer::_utmProjection;

MapUSGSImageLayer::MapUSGSImageLayer(TransactionManager* tmgr, wxString name):
        MapObjectContainer(tmgr,name),
        _scale(10),
        _type(ThemeType_Urban)
{
}

MapUSGSImageLayer::~MapUSGSImageLayer() {
        CancelPendingRequests();
}

void MapUSGSImageLayer::FetchImagesForRectangle(const TDRectangle geoBounds,
                                                wxFileName imgPath)
{
        MapUSGSImageList images;
        float tilesize=GetTilesizeForScale(_scale);

        UTMProjection proj;
        proj.Init(geoBounds.GetCenter());
        int zone=proj.GetZone();
        
        TDRectangle utmRect;
        
        utmRect=TDRectangle(proj.ConvertToLocal(geoBounds.GetTopLeft()),
                            proj.ConvertToLocal(geoBounds.GetBottomRight()));
        
        int xmin,xmax,ymin,ymax;
        
        xmin=(int)floor(utmRect.GetLeft()  /tilesize);
        xmax=(int) ceil(utmRect.GetRight() /tilesize);
        ymin=(int)floor(utmRect.GetBottom()/tilesize);
        ymax=(int) ceil(utmRect.GetTop()   /tilesize);

        imgPath.Mkdir(0755,wxPATH_MKDIR_FULL);
        
        for (int x=xmin;x<xmax;x++) {
                for (int y=ymin;y<ymax;y++) {
                        wxString filename;
                        filename.Printf(wxT("T%d_S%d_X%d_Y%d_Z%d"),_type,_scale,x,y,zone);
                        
                        wxFileName fullFile=imgPath;
                        fullFile.SetName(filename);
                        fullFile.SetExt(wxT("jpg"));
                        
                        wxString fullName=fullFile.GetFullPath();
                        
                        if (TRANSACTIONOBJECT_STATEVAR(_imagesByPath).find(fullName)!=TRANSACTIONOBJECT_STATEVAR(_imagesByPath).end()) {
                                // Don't reload images that are already shown
                                continue;
                        }

                        MapUSGSImage::SmartRef imageObject;
                        imageObject=new MapUSGSImage(GetTransactionManager(),_type,_scale,x,y,zone);
                        imageObject->SetFilename(fullFile.GetFullPath());
                        AttachObject(imageObject);
                        
                        BeforeWrite();
                        TRANSACTIONOBJECT_STATEVAR(_imagesByPath)[fullName]=imageObject;
                        AfterWrite();
                        
                        if (!fullFile.FileExists()) {
                                // Only download images for which don't have the data yet
                                images.Append(imageObject);
                        }
                }
        }
        USGSFetcher::GetInstance()->AddRequests(images);
}

void MapUSGSImageLayer::CancelPendingRequests() {
        // cancel all requests still pending
        MapUSGSImageList imagelist;
        MapObjectList::Node* node;
        for (node=_objects.GetFirst();node;node=node->GetNext()) {
                MapUSGSImage* img=wxDynamicCast(node->GetData(),MapUSGSImage);
                if (!img)
                        continue;
                imagelist.Append(img);
        }
        USGSFetcher::GetInstance()->CancelRequests(imagelist);
}

Point3D MapUSGSImageLayer::GetNorthWestPosForTile(int zone, int scale, int x, int y) {
        static bool initialised=false;
        if (!initialised) {
                _utmProjection.Init(Point3D());
                initialised=true;
        }
        float tilesize=GetTilesizeForScale(scale);
        TDRectangle utmRect=TDRectangle((y+1)*tilesize,y*tilesize,
                                    (x+1)*tilesize,x*tilesize);
        Point3D pos=_utmProjection.ConvertFromLocal(utmRect.GetTopLeft(),zone);
        return pos;
}

bool MapUSGSImageLayer::IsThemeScaleAvailable(ThemeType type, int scale) {
        if (scale<0)
                return false;
        
        switch (type) {
        case ThemeType_DOQ:
                return (scale>=10 && scale<=17);
        case ThemeType_DRG:
                return (scale>=11 && scale<=17);
        case ThemeType_Urban:
                return (scale>=8 && scale<=17);
        }
}

void MapUSGSImageLayer::DetachObject(MapObject* obj) {
        MapUSGSImage* img=wxDynamicCast(obj,MapUSGSImage);
        if (img) {
                BeforeWrite();
                TRANSACTIONOBJECT_STATEVAR(_imagesByPath).erase(img->GetFilename());
                AfterWrite();
        }
        MapObjectContainer::DetachObject(obj);
}

