#ifndef USGSFetcher_h
#define USGSFetcher_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

#include <wx/filename.h>

#include "MapUSGSImage.h"

#include <Math/TDRectangle.h>

#include <curl/curl.h>

#include "USGSProgressListener.h"

WX_DECLARE_LIST(USGSProgressListener,USGSProgressListenerList);

DECLARE_EVENT_TYPE(wxEVT_USGS_TILE_LOADED,-1);

class USGSTileLoadedEvent: public wxEvent {
        
        DECLARE_CLASS(USGSTileLoadedEvent)
        
public:
	USGSTileLoadedEvent(wxEventType eventType,
                            MapUSGSImage::SmartRef image,
                            unsigned int pendingRequests,
                            unsigned int totalRequests):
                        wxEvent(0,eventType),
                        _image(image),
                        _pendingRequests(pendingRequests),
                        _totalRequests(totalRequests)
        {
        }
        
        USGSTileLoadedEvent(const USGSTileLoadedEvent& evt):
                wxEvent(evt),
                _image(evt._image),
                _pendingRequests(evt._pendingRequests),
                _totalRequests(evt._totalRequests)
        {
        }
        
        virtual wxEvent* Clone() const { return new USGSTileLoadedEvent(*this); }
        
        MapUSGSImage::SmartRef GetImage() const { return _image; }
        unsigned int GetPendingRequests() const { return _pendingRequests; }
        unsigned int GetTotalRequests() const { return _totalRequests; }
protected:
        MapUSGSImage::SmartRef _image;
        unsigned int _pendingRequests;
        unsigned int _totalRequests;
};

class USGSFetcher: public wxEvtHandler, public wxThread {
        
        DECLARE_CLASS(USGSFetcher)
        
public:
        static USGSFetcher* GetInstance();
        
        virtual ~USGSFetcher();
        virtual void* Entry();
        
        void AddRequests(const MapUSGSImageList& images);
        void CancelRequests(const MapUSGSImageList& images);
        void CancelPendingRequests();
        
        void RegisterProgressListener(USGSProgressListener* listener);
        void UnregisterProgressListener(USGSProgressListener* listener);
        void FireUpdateProgress(unsigned int pending, unsigned int total);
protected:
	USGSFetcher();
        
        void ProcessRequest(MapUSGSImage::SmartRef image);
        
        void OnTileLoaded(USGSTileLoadedEvent& evt);
        
        static USGSFetcher* _singletonInstance;
        
        wxCriticalSection _queueAccess;
        wxSemaphore _queueSema;
        MapUSGSImageList _queue;
        unsigned int _pendingRequests,_totalRequests;
        
        USGSProgressListenerList _progressListeners;
        
        DECLARE_EVENT_TABLE()
};

typedef void (wxEvtHandler::*USGSTileLoadedEventFunction)(USGSTileLoadedEvent&);

#define EVT_USGS_TILE_LOADED(fn) \
    DECLARE_EVENT_TABLE_ENTRY( wxEVT_USGS_TILE_LOADED, wxID_ANY, -1, \
    (wxObjectEventFunction) (wxEventFunction) (USGSTileLoadedEventFunction) & fn ,\
    (wxObject *) NULL ),

#endif

