#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "MapObject.h"

#include "MapCoordinateSystem.h"
#include "MapEditor.h"
#include "MapObjectContainer.h"

#include <cassert>
#include <wx/listimpl.cpp>

WX_DEFINE_LIST(MapObjectList);

IMPLEMENT_CLASS(MapObject,TransactionObject)

const MapPass MapObject::StandardPass(wxT("Main map"));

MapObject::MapObject(TransactionManager* transactionManager):
        TransactionObject(transactionManager),
        _container(NULL)
{
}

MapObject::~MapObject() {
        wxASSERT_MSG(GetContainer()!=NULL, wxT("Object is destructed while still attached to a container"));
}

TDRectangle MapObject::GetLocalBounds(MapEditor* editor) {
        MapCoordinateSystem& cs=editor->GetCoordinateSystem();
        wxRect devbounds=GetDeviceBounds(editor);
	
	if (devbounds.IsEmpty()) {
		return TDRectangle();
	}
        
        Point3D localTopLeft=cs.DeviceToLogical(devbounds.GetTopLeft());
        Point3D localBottomRight=cs.DeviceToLogical(devbounds.GetBottomRight());
        
        return TDRectangle(localTopLeft,localBottomRight);
}

void MapObject::Damage() {
        if (GetContainer()==NULL)
                return;
        GetContainer()->Damage(this);
}

void MapObject::RemoveFromContainer() {
        wxASSERT_MSG(GetContainer()!=NULL, wxT("Object is not attached to a container"));
        MapLayerEditors editors=GetContainer()->GetEditors();
        MapLayerEditors::Node *editorNode;
        
        // FIXME: we might want to optimise this by keeping a list of active selections
        for (editorNode=editors.GetFirst();editorNode!=NULL;editorNode=editorNode->GetNext()) {
                MapEditor* editor=editorNode->GetData();
                MapSelectionManager& selectionManager=editor->GetSelectionManager();
                MapSelection* selection=selectionManager.GetSelectionForObject(this);
                if (selection!=NULL) {
                        selectionManager.RemoveSelection(selection);
                }
        }
        
        GetContainer()->DetachObject(this);
}

void MapObject::RestoreMemento(wxObject* object) {
        TransactionObject::RestoreMemento(object);
        Memento* memento=dynamic_cast<Memento*>(object);
        wxASSERT(memento);
        
        Damage(); /* Force redraw in old boundaries */
        _container=memento->_container;
        Damage(); /* Force redraw in new boundaries */
}

