#ifndef MapPass_h
#define MapPass_h

// For compilers that support precompilation, includes "wx.h".
#include <wx/wxprec.h>

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
// Include your minimal set of headers here, or wx.h
#include <wx/wx.h>
#endif

/**
 * \brief A MapPass represents a painting pass.
 *
 * It is used, e.g., to differentiate between lighting and pavement,
 * allowing to always draw lighting on top of pavement as well as
 * disable either lighting or pavement painting.
 *
 * Passes can only be equal by identity, i.e., they are compared by their
 * address. The intention is to allow declaration of passes by simply providing
 * a static MapPass instance.
 *
 * \sa MapPassStandard
 */
class MapPass {
public:
        MapPass(const wxString& name):_name(name) {}
        
        /**
         * \brief Retrieve the name of the pass.
         */
        const wxString& GetName() const { return _name; }
        
        bool operator==(const MapPass& p) const { return &p==this; }
        bool operator!=(const MapPass& p) const { return &p!=this; }
protected:
        wxString _name;
};

#endif
