#include <algorithm>
#include <cmath>

template<typename T> T norm(T v);

#ifdef HAVE_FABSL
template<> long double norm<long double>(long double v) { return fabsl(v); }
#endif
template<> double norm<double>(double v) { return fabs(v); }
template<> float norm<float>(float v) { return fabsf(v); }

/* Solve generic linear equations */    
template <typename T, int n>
void solveLinear(T A[n][n+1], T x[n]) {
        int order[n];
        T pivotval;
        int pivotcol,pivotrow;
        for (int i=0;i<n;i++)
                order[i]=i;
        
        /* diagonalise */
        for (int i=0;i<n-1;i++) {
                /* find pivot */
                pivotcol=pivotrow=i;
                pivotval=A[i][i];
                for (int j=i;j<n;j++) {
                        for (int k=i;k<n;k++) {
                                if (norm(A[j][k])>norm(pivotval)) {
                                        pivotrow=j;
                                        pivotcol=k;
                                        pivotval=A[j][k];
                                }
                        }
                }

                /* swap pivot to A[i][i] */
                for (int j=0;j<n+1;j++)
                        std::swap(A[i][j],A[pivotrow][j]);
                for (int j=0;j<n;j++)
                        std::swap(A[j][i],A[j][pivotcol]);
                std::swap(order[i],order[pivotcol]);
                
                /* eliminate */
                for (int j=i+1;j<n;j++)
                        for (int k=n;k>=i;k--)
                                A[j][k]=A[j][k]*A[i][i]-A[i][k]*A[j][i];
        }

        /* solve */
        for (int i=n-1;i>=0;i--) {
                T sum=A[i][n];
                for (int j=i+1;j<n;j++)
                        sum-=A[i][j]*x[order[j]];
                x[order[i]]=sum/A[i][i];
        }
}
