#if HAVE_CONFIG_H
#  include <config.h>
#endif /* HAVE_CONFIG_H */

#include "PanZoomMode.h"
#include <Map/MapEditor.h>
#include <Map/MapCoordinateSystem.h>
#include <Map/MapSelectionManager.h>
#include <Math/point3d.h>
#include <Math/TDRectangle.h>
#include <algorithm>

IMPLEMENT_CLASS(PanZoomMode,Mode)

BEGIN_EVENT_TABLE(PanZoomMode,Mode)
        EVT_MOTION(PanZoomMode::OnMouseMotion)
        EVT_MOUSEWHEEL(PanZoomMode::OnMouseWheel)
        EVT_LEFT_DOWN(PanZoomMode::CatchMouseEvent)
        EVT_LEFT_UP(PanZoomMode::CatchMouseEvent)
END_EVENT_TABLE()

PanZoomMode::PanZoomMode():
        _mouseWheelRotation(0) {
}

bool PanZoomMode::CanExit() {
        return false;
}

void PanZoomMode::OnMouseMotion(wxMouseEvent& evt) {
        if (!evt.Dragging() || !evt.ControlDown()) {
                _lastPosition=evt.GetPosition();
                if (!evt.ControlDown())
                        evt.Skip();
                return;
        }
        wxPoint curpos=evt.GetPosition();
        wxSize devDelta=wxSize(_lastPosition.x-curpos.x,
                               _lastPosition.y-curpos.y);
        
        GetEditor()->Scroll(devDelta);
        _lastPosition=curpos;
}

void PanZoomMode::OnMouseWheel(wxMouseEvent& evt) {
        _mouseWheelRotation+=evt.GetWheelRotation();
        int delta=evt.GetWheelDelta();
        
        // workaround for wxgtk2.4 bug on Debian stable
        if (delta==0) delta=120;
        
        int steps=_mouseWheelRotation/delta;
        _mouseWheelRotation-=steps*delta;
        
        wxPoint curpos=evt.GetPosition();
        Point3D localCenter=GetEditor()->GetCoordinateSystem().DeviceToLogical(curpos);
        
        GetEditor()->Zoom(localCenter,steps);
}

void PanZoomMode::CatchMouseEvent(wxMouseEvent& evt) {
        _lastPosition=evt.GetPosition();
        if (!evt.ControlDown())
                evt.Skip();
}

